from flask import Flask, render_template, request, redirect, url_for, flash, session
from flask_sqlalchemy import SQLAlchemy
from flask_basicauth import BasicAuth

app = Flask(__name__)
app.secret_key = 'fsdfsfsd52352'  # set the secret key here
app.config['BASIC_AUTH_USERNAME'] = 'root'  # 设置用户名
app.config['BASIC_AUTH_PASSWORD'] = '123456'  # 设置密码
basic_auth = BasicAuth(app)

DB_URI = 'mysql+pymysql://c22090044:981128qwER@csmysql.cs.cf.ac.uk:3306/c22090044_c22090044'


app.config['SQLALCHEMY_DATABASE_URI'] = DB_URI

db = SQLAlchemy(app)


class Questions(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    question = db.Column(db.String(255), nullable=False)
    option1 = db.Column(db.String(255), nullable=False)
    option2 = db.Column(db.String(255), nullable=False)
    option3 = db.Column(db.String(255), nullable=False)
    option4 = db.Column(db.String(255), nullable=False)
    answer = db.Column(db.String(255), nullable=False)
    feedback = db.Column(db.String(255))

    def delete(self):
        db.session.delete(self)
        db.session.commit()


@app.route('/')
def index():
    return render_template("index.html")


@app.route('/questions')
def questions():
    questions = Questions.query.all()
    return render_template("questions.html", questions=questions)


@app.route('/process_form', methods=['GET', 'POST'])
def process_form():
    test_type = request.form['test_type']
    session['test_type'] = test_type
    return render_template("index.html", test_type=test_type)


@app.route('/test')
def test():
    questions = []
    test_type = session.get('test_type')
    test_questions = Test.query.all()
    for test_question in test_questions:
        question = Questions.query.get(test_question.question_id)
        if question:
            questions.append(question)
    session['question_ids'] = [question.id for question in questions]
    return render_template('test.html', questions=questions, test_type=test_type)


@app.route('/selectquestion')
@basic_auth.required
def selectquestion():
    questions = Questions.query.all()
    return render_template("selectquestion.html", questions=questions)


@app.route('/add_question', methods=['GET', 'POST'])
def add_question():
    if request.method == 'POST':
        question = request.form['question']
        option1 = request.form['option1']
        option2 = request.form['option2']
        option3 = request.form['option3']
        option4 = request.form['option4']
        answer = request.form['answer']
        feedback = request.form['feedback']
        new_question = Questions(question=question, option1=option1, option2=option2,
                                 option3=option3, option4=option4, answer=answer, feedback=feedback)
        db.session.add(new_question)
        db.session.commit()
        return redirect(url_for('questions'))
    return render_template("add_question.html")


class Test(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    question_id = db.Column(db.Integer, db.ForeignKey('questions.id'))
    question = db.relationship('Questions', backref='tests', lazy=True)


@app.route('/add_to_test', methods=['POST'])
def add_to_test():
    question_ids = request.form.getlist('questions')
    if len(question_ids) == 0:
        flash('Please select at least one question to add to the test.')
        return redirect(url_for('selectquestion'))

    # 删除 test 表的所有数据
    Test.query.delete()
    db.session.commit()
    questions = []
    for question_id in question_ids:
        test = Test(question_id=question_id)
        question = Questions.query.get(question_id)
        if question:
            questions.append(question)
        db.session.add(test)
    db.session.commit()
    return render_template('createTest.html', questions=questions)


@app.route('/delete_question/<int:id>', methods=['POST'])
def delete_question(id):
    question = Questions.query.get(id)
    question.delete()
    return redirect(url_for('questions'))


@app.route('/result', methods=['GET', 'POST'])
def result():
    # get the answers submitted by the user from the form
    user_answers = {}
    questions = []
    question_ids = session.get('question_ids')
    for question_id in question_ids:
        question = Questions.query.get(question_id)
        user_answers[int(question_id)] = request.form.get(
            'answer_' + str(question_id))
        questions.append(question)

    # for key in request.form.keys():
    #     if key.startswith('answer_'):
    #         question_id = key.split('_')[1]
    #         user_answers[int(question_id)] = request.form[key]

    # get the correct answers for the questions
    correct_answers = {question.id: question.answer for question in questions}

    # calculate the score
    score = 0
    for question_id, user_answer in user_answers.items():
        if user_answer == correct_answers[question_id]:
            score += 1

    # render the result template and pass in the score and the questions
    return render_template('testresult.html', score=score, questions=questions, user_answers=user_answers)


if __name__ == '__main__':
    app.run(debug=True)
